<?php

namespace Modules\Pp\Http\Controllers;

use Illuminate\Contracts\Support\Renderable;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Modules\Pp\Entities\Product;
use Modules\Pp\Entities\BomHeader;
use Modules\Pp\Entities\BomData;

class BillOfMaterialController extends Controller
{

/**
     * searchName
     * This function is used to search employee name from product id.
     * @return Json This returns as response.
     */
    public function searchPName()
    {
        //$data = Product::all();
        $searchAray = [];
        $data = Product::where('product_type_id', '=', 3 )->get();
        for ($j = 0; $j < count($data); $j++) {
            array_push($searchAray, [
                "img" => "",
                "id" => $data[$j]['product_id'],
                "value" => $data[$j]['product_name'],
            ]);
        }
        $data = Product::where('product_type_id', '=', 4 )->get();
        for ($j = 0; $j < count($data); $j++) {
            array_push($searchAray, [
                "img" => "",
                "id" => $data[$j]['product_id'],
                "value" => $data[$j]['product_name'],
            ]);
        }
        return response()->json($searchAray);
    }


    public function getPName($product_id)
    {
        $data = Product::find($product_id);
        return response()->json($data);
    }



    /**
     * save
     * This function is used to save new BOM to database.
     * @param request This is the paramter to save function
     * @return Json This returns as response.
     */
    public function save(Request $request)
    {

        try {

            $bomHeader = new BomHeader();
            $bomHeader->product_id = $request->get('txtProductID');;
            $bomHeader->product_name = $request->get('txtProductName');
            $bomHeader->quantity = $request->get('txtQuantity');
            $bomHeader->uom = $request->get('txtuom');
            $bomHeader->description = $request->get('txtDescription');
            // $bomHeader->status = true;   //correct this later nipuna
            $save = $bomHeader->save();

            if ($save) {
                $responseBody = $this->responseBody(true, "BomHeader", "saved", null);
            }
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "error", "error", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }


    /**
     * responseBody
     * This is used to create response.
     * @param success This is the paramter require boolean
     * @param name This is the paramter require ui table name
     * @param message This is the paramter require message content
     * @param result This is the paramter require result as some of data to return client
     * @return Json This returns as response.
     */
    private function responseBody($success, $name, $message, $result)
    {
        $body = [
            "success" => $success,
            "message" => $message,
            "name" => $name,
            "result" => $result
        ];
        return $body;
    }


    /**
     * allbomHeader
     * This function is used to search all BOM records from the database.
     * @return Json This returns as response.
     */
    public function allbomHeader()
    {

        try {
            $bomHeader =  BomHeader::orderBy('product_id')->get();
            $responseBody = $this->responseBody(true, "BomHeader", "all", $bomHeader);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "error", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }


    /**
     * deleteBom
     * This function is used to delete BOM from bill of material header id.
     * @return Json This returns as response.
     */
    public function deleteBom($id)
    {
        try {
            $bomHeader = BomHeader::find($id);
            $delete = $bomHeader->delete();
            if ($delete) {
                $responseBody = $this->responseBody(true, "BomHeader", "deleted", null);
            }
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "error", "error", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }


    /**
     * bomHeader
     * This function is used to search BOM from bill of material header id.
     * @return Json This returns as response.
     */
    public function bomHeader($id)
    {

        try {
            $bomHeader = BomHeader::find($id);
            $product_id = $bomHeader->product_id;

            $responseBody = $this->responseBody(true, "CustomerPlan", "all", ["BomHeader" => $bomHeader]);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "error", "error", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }

     /**
     * updateBomHeader
     * This function is used to update bill of material header.
     * @param request This is the paramter to update function
     * @return Json This returns as response.
     */
    public function updateBomHeader(Request $request)
    {

        try {


           $id = $request->get('hidBomID');
           $bomHeader = BomHeader::find($id);
           $bomHeader->product_id = $request->get('txtProductID');;
           $bomHeader->product_name = $request->get('txtProductName');
           $bomHeader->quantity = $request->get('txtQuantity');
           $bomHeader->uom = $request->get('txtuom');
           $bomHeader->description = $request->get('txtDescription');
           $save = $bomHeader->save();

            if ($save) {
                $responseBody = $this->responseBody(true, "BomHeader", "updated",  $bomHeader);
            }
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "error", $bomHeader, $exception);
        }
        return response()->json(["data" => $responseBody]);
    }


    /**
     * disableBomHeader
     * This function is used to disable BOM header records.
     * @param request This is the paramter to disableBomHeader function
     * @param id This is the paramter for identify update record.
     */
    public function disableBomHeader(Request $request, $id)
    {
        try {

            $value = $request->get('status');

            $object = BomHeader::find($id);
            $object->status = $value;
            $update = $object->save();
            if ($update) {
                $responseBody = $this->responseBody(true, "BomHeader", "updated", null);
            }
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "BomHeader", "error", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }


    /**
     * searchName
     * This function is used to search employee name from product id.
     * @return Json This returns as response.
     */
    public function searchPDName()
    {
        //$data = Product::all();
        $searchAray = [];
        $data = Product::where('product_type_id', '=', 1 )->get();
        for ($j = 0; $j < count($data); $j++) {
            array_push($searchAray, [
                "img" => "",
                "id" => $data[$j]['product_id'],
                "value" => $data[$j]['product_name'],
            ]);
        }
        $data = Product::where('product_type_id', '=', 2 )->get();
        for ($j = 0; $j < count($data); $j++) {
            array_push($searchAray, [
                "img" => "",
                "id" => $data[$j]['product_id'],
                "value" => $data[$j]['product_name'],
            ]);
        }
        return response()->json($searchAray);
    }


    public function getPDName($product_id)
    {
        $data = Product::find($product_id);
        return response()->json($data);
    }



    /**
     * saveData
     * This function is used to save new BOM data to database.
     * @param request This is the paramter to save function
     * @return Json This returns as response.
     */
    public function saveData(Request $request)
    {

        try {

            $bomData = new BomData();
            $bomData->pp_bom_header_id = $request->get('txtBomHeaderId');
            //$bomData->pp_bom_header_id =1; //correct this late check the upper commented part nipuna

            $bomData->product_id = $request->get('txtProductID2');
            $bomData->product_name = $request->get('txtProductName2');
            $bomData->quantity = $request->get('txtQuantity2');
            $bomData->uom = $request->get('txtUom2');
            $bomData->tolerance = $request->get('txtTolerance');
            $bomData->status = true;
            $save = $bomData->save();

            if ($save) {
                $responseBody = $this->responseBody(true, "BomData", "saved", $bomData);
            }
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "error", "error", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }


    /**
     * allbomData
     * This function is used to search all BOM data records from the database.
     * @return Json This returns as response.
     */
    public function allbomData($id)
    {

        try {
            $bomData = BomData::find($id);
            $bomData =  BomData::where('pp_bom_header_id', $id)->get();
            $responseBody = $this->responseBody(true, "BomData", "all", $bomData);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "error", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }


    /**
     * deletebomData
     * This function is used to delete BOM data from BOM plan data id.
     * @return Json This returns as response.
     */
    public function deletebomData($id)
    {
        try {
            $bomData = BomData::find($id);
            $deleteData = $bomData->delete();
            if ($deleteData) {
                $responseBody = $this->responseBody(true, "BomData", "deleted", $bomData);
            }
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "error", "error", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }


    /**
     * bomData
     * This function is used to search leave from BOM pla data id.
     * @return Json This returns as response.
     */
    public function bomData($id)
    {

        try {
            $bomData = BomData::find($id);



            $responseBody = $this->responseBody(true, "bomData", "all", ["bomData" => $bomData]);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "error", "error", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }



    /**
     * updateBomData
     * This function is used to update BOM plan data.
     * @param request This is the paramter to update function
     * @return Json This returns as response.
     */
    public function updateBomData(Request $request)
    {

        try {

            $id = $request->get('hidBomDataID');
            $bomData = BomData::find($id);
            $bomData->pp_bom_header_id = $request->get('txtBomHeaderId');
            $bomData->product_id = $request->get('txtProductID2');
            $bomData->product_name = $request->get('txtProductName2');
            $bomData->quantity = $request->get('txtQuantity2');
            $bomData->uom = $request->get('txtUom2');
            $bomData->tolerance = $request->get('txtTolerance');
            $save = $bomData->save();

            if ($save) {
                $responseBody = $this->responseBody(true, "BomData", "updated", "");
            }
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "error", $bomData, $exception);
        }
        return response()->json(["data" => $responseBody]);
    }



    /**
     * disableBomData
     * This function is used to disable BOM plan data records.
     * @param request This is the paramter to disableBomData function
     * @param id This is the paramter for identify update record.
     */
    public function disableBomData(Request $request, $id)
    {
        try {

            $value = $request->get('status');

            $object = BomData::find($id);
            $object->status = $value;
            $update = $object->save();
            if ($update) {
                $responseBody = $this->responseBody(true, "BomData", "updated", null);
            }
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "BomData", "error", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }



}
